<?php

namespace App\domain\worldpay;

use App\Models\WorldpayPayments;
use App\Traits\FileStorageTrait;
use Exception;
use GuzzleHttp\Client;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Str;

class ProcessPaymentOrderAction
{
   use FileStorageTrait;
   protected Client $client;
   protected string $merchantCode;
   protected string $installationId;
   protected string $credentials;
   protected string $paymentOrderId;
   protected array $requesterDetails;
   protected array $orderDetails;
   public function __construct()
   {
      $caCertPath = base_path('cacert.pem');

      $this->client = new Client([
         'verify' => $caCertPath,
      ]);

      $this->merchantCode = 'CARLTONFLIGHTSECOM';
      $this->installationId = '1499491';
      $username = '9UHHHZ6PRZQ6QX24LEXX';
      $password = 'Carlton@@2024';
      $this->credentials = base64_encode("$username:$password");

      $this->paymentOrderId = Str::uuid();
   }

   public function execute(array $orderDetails, array $requesterDetails)
   {
      try {
         $this->requesterDetails = $requesterDetails;
         $this->orderDetails = $orderDetails;

         $this->createPaymentOrderRecord();

         $xmlContent = $this->buildXmlRequest(
            $this->paymentOrderId,
            $orderDetails['amount'],
            $orderDetails['currency'],
            $orderDetails['shopperEmail'],
            $orderDetails['address']
         );

         $xmlHeader = '<?xml version="1.0" encoding="UTF-8"?>';
         $xmlDoctype = '<!DOCTYPE paymentService PUBLIC "-//Worldpay//DTD Worldpay PaymentService v1//EN" "http://dtd.worldpay.com/paymentService_v1.dtd">';
         $xmlString = $xmlHeader . "\n" . $xmlDoctype . "\n" . $xmlContent;

         self::storeInFile('xml', 'worldpay-payment-order-request', $xmlString, 'worldpay/');

         $response = $this->client->post($this->getEndPointUrl(), [
            'body' => $xmlString,
            'headers' => [
               'Content-Type' => 'text/xml',
               'Authorization' => 'Basic ' . $this->credentials,
            ]
         ]);

         self::storeInFile('xml', 'worldpay-payment-order-reply', $response->getBody(), 'worldpay/');

         Log::info('Payment Order response Header-->' , $response->getHeaders());
         Log::info('Payment Order response Body-->' . $response->getBody());

         return [
            "paymentPageURL" => $this->extractUrlFromXmlResponse($response->getBody()),
            "paymentOrderId" => $this->paymentOrderId
         ];
      } catch (Exception $e) {
         throw new Exception('Payment order failed due to ' . $e->getMessage());
      }
   }

   protected function createPaymentOrderRecord()
   {
      $worldpayPayment = new WorldpayPayments();
      $worldpayPayment->payment_order_id =  $this->paymentOrderId;
      $worldpayPayment->frontend_url =  $this->requesterDetails['redirectURL'];
      $worldpayPayment->source =  $this->requesterDetails['source'];
      $worldpayPayment->pnr_id =  $this->orderDetails['pnrId'];
      $worldpayPayment->shopper_email =  $this->orderDetails['shopperEmail'];
      $worldpayPayment->currency =  $this->orderDetails['currency'];
      $worldpayPayment->amount =  $this->orderDetails['amount'];
      $worldpayPayment->status =  PaymentStatus::PENDING;
      $worldpayPayment->save();
   }

   protected function extractUrlFromXmlResponse($xmlResponse)
   {
      $xml = new \SimpleXMLElement($xmlResponse);
      $namespace = $xml->getNamespaces(true);

      // Check if there are any errors using XPath
      $errorNodes = $xml->xpath('//reply/error');
      if (!empty($errorNodes)) {
         // Extract the error message
         $errorMessage = (string) $errorNodes[0];

         // Throw a new exception with the error message
         throw new Exception($errorMessage);
      }

      // Extract the reference URL using XPath
      $referenceNodes = $xml->xpath('//reference');
      if (empty($referenceNodes)) {
         throw new Exception('Reference URL not found in the response.');
      }

      $url = (string) $referenceNodes[0];
      return $url;
   }


   protected function buildXMLRequest($orderCode, $amount, $currency, $shopperEmail, $address): string
   {
      $xml = new \SimpleXMLElement('<paymentService version="1.4" merchantCode="' . $this->merchantCode . '"/>');

      $amount = str_replace('.', '', $amount);

      $submit = $xml->addChild('submit');
      $order = $submit->addChild('order');
      $order->addAttribute('orderCode', $orderCode);
      $order->addAttribute('captureDelay', 1);
      $order->addAttribute('installationId', $this->installationId);

      $order->addChild('description', 'First payment testing');
      $amountElement = $order->addChild('amount');
      $amountElement->addAttribute('currencyCode', $currency);
      $amountElement->addAttribute('exponent', '2');
      $amountElement->addAttribute('value', $amount);

      $order->addChild('orderContent', '<![CDATA[Your detailed order content here]]>');
      $paymentMethodMask = $order->addChild('paymentMethodMask');
      $paymentMethodMask->addChild('include')->addAttribute('code', 'ALL');

      $shopper = $order->addChild('shopper');
      $shopper->addChild('shopperEmailAddress', $shopperEmail);

      $shippingAddress = $order->addChild('shippingAddress');
      $this->addAddress($shippingAddress, $address);

      $billingAddress = $order->addChild('billingAddress');
      $this->addAddress($billingAddress, $address);

      $dom = new \DOMDocument('1.0', 'UTF-8');
      $dom->preserveWhiteSpace = false;
      $dom->formatOutput = true;
      $dom->loadXML($xml->asXML());

      $xmlContentWithoutDeclaration = $dom->saveXML($dom->documentElement);

      return $xmlContentWithoutDeclaration;
   }

   protected function addAddress($parentElement, $address)
   {
      $addressElement = $parentElement->addChild('address');
      $addressElement->addChild('address1', $address['address1']);
      $addressElement->addChild('address2', $address['address2']);
      $addressElement->addChild('address3', $address['address3']);
      $addressElement->addChild('postalCode', $address['postalCode']);
      $addressElement->addChild('city', $address['city']);
      $addressElement->addChild('state', $address['state']);
      $addressElement->addChild('countryCode', $address['countryCode']);
   }



   protected function getEndPointUrl(): string
   {
      return 'https://secure-test.worldpay.com/jsp/merchant/xml/paymentService.jsp';
   }
}
